export default (capacity, ctr = Array) => {
    const buffer = new ctr(capacity)
    let head = 0
    let tail = 0
    let size = 0

    return Object.freeze({

        enqueue(item) {
            if (this.isFull()) {
                tail = (tail + 1) % capacity
            }

            buffer[head] = item
            head = (head + 1) % capacity
            size = Math.min(capacity, size + 1)
        },
        dequeue() {
            if (this.isEmpty()) {
                return undefined
            }

            const item = buffer[tail]
            tail = (tail + 1) % capacity
            size--
            return item
        },

        peek() {
            if (this.isEmpty()) {
                return undefined
            }

            return buffer[tail]
        },

        isFull: () => capacity === size,
        isEmpty: () => size === 0,
        size() { return size },

        valueOf() {
            const result = []

            for (const x of this) {
                result.push(x)
            }

            return result
        },

        clear() {
            head = 0
            tail = 0
            size = 0
        },

        *[Symbol.iterator]() {
            let i = tail
            for (let j = 0; j < size; j++) {
                yield buffer[i]
                i = (i + 1) % capacity
            }
        }
    })
}

